"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsonApiAxiosError = void 0;
const json_api_error_1 = require("./json-api-error");
/**
 * Axios API error
 *
 * Creates a Axios JSON API error response
 *
 * @see http://jsonapi.org/
 */
const jsonApiAxiosError = (err) => {
    const response = {
        status: 500,
        title: "Unknown Error",
        details: "An unknown error occurred",
        code: json_api_error_1.ERROR_CODE_ENUM.UNKNOWN_ERROR
    };
    // If the domain or endpoint is not found
    if (err.code === "ENOTFOUND" || err.code === "EPROTO" || err.code === "ECONNREFUSED") {
        return json_api_error_1.jsonApiError({
            code: json_api_error_1.ERROR_CODE_ENUM.API_CONNECTION_ERROR,
            status: 404,
            source: { stack: err.stack },
            title: "Endpoint not found",
            details: err.message
        });
    }
    // Below catch any unexpected error which has a 400+ error code
    if (err.response && err.response.status >= 400) {
        // Set the error response
        let errorMessage = `${err.message}`;
        errorMessage += err.response && err.response.data.errors && err.response.data ? `: ${err.response.data.errors}` : "";
        return json_api_error_1.jsonApiError({
            code: json_api_error_1.ERROR_CODE_ENUM.API_ERROR,
            status: err.response.status,
            source: { stack: err.stack },
            title: "API request failure",
            details: errorMessage
        });
    }
    // Assume that the 'err' is already of type JSONApiResponse.
    return json_api_error_1.jsonApiError(response);
};
exports.jsonApiAxiosError = jsonApiAxiosError;
