import * as Joi from "joi";
export declare const jsonAPIErrorJoiSchema: Joi.ObjectSchema<JSONApiErrorJSON>;
export declare class JSONApiErrorStatic extends Error {
    id?: string;
    title: string;
    status: number;
    code: ERROR_CODE_ENUM;
    /**  an object containing references to the source of the error, optionally including any of the following members: */
    source?: {
        pointer?: string;
        parameter?: string;
        stack?: string;
    };
    meta?: JSONApiMeta;
    /**
     * Check if a plain object/ variable meets the JSONApiError compliance.
     *
     * @param error: the error object to check.
     */
    static isJSONApiError(error: Record<string, any>): boolean;
    constructor(error: JSONApiErrorJSON | Joi.ValidationError | Partial<Error> | undefined);
    toJson(): JSONApiErrorJSON;
}
/**
 * Creates and API
 *
 * Creates a JSON API error response
 *
 * @see http://jsonapi.org/
 * @param error - The suppose JSON API error object
 * @return A JSON API Response containing our error(s)
 */
export declare const jsonApiError: (errorInfo: JSONApiErrorJSON | Joi.ValidationError | Error | undefined) => JSONApiErrorStatic;
export interface JSONApiMeta {
    environment?: string;
    resource?: string;
    url?: string;
    httpMethod?: string;
    functionName?: string;
}
export declare enum ERROR_CODE_ENUM {
    API_ERROR = "API_ERROR",
    API_CONNECTION_ERROR = "API_CONNECTION_ERROR",
    AUTHENTICATION_ERROR = "AUTHENTICATION_ERROR",
    INVALID_DATA_ERROR = "INVALID_DATA_ERROR",
    DATABASE_ERROR = "DATABASE_ERROR",
    NOT_FOUND_ERROR = "NOT_FOUND_ERROR",
    THIRD_PARTY_ERROR = "THIRD_PARTY_ERROR",
    UNKNOWN_ERROR = "UNKNOWN_ERROR"
}
export interface JSONApiErrorJSON {
    id?: string;
    title: string;
    details: string;
    status: number;
    code: ERROR_CODE_ENUM;
    /**  an object containing references to the source of the error, optionally including any of the following members: */
    source?: {
        pointer?: string;
        parameter?: string;
        stack?: string;
    };
    meta?: JSONApiMeta;
}
