"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ERROR_CODE_ENUM = exports.jsonApiError = exports.JSONApiErrorStatic = exports.jsonAPIErrorJoiSchema = void 0;
const uuid = __importStar(require("uuid"));
const Joi = __importStar(require("joi"));
const R = __importStar(require("ramda"));
const Sentry = __importStar(require("@sentry/node"));
exports.jsonAPIErrorJoiSchema = Joi.object({
    id: Joi.string().optional(),
    title: Joi.string().required(),
    details: Joi.string().required(),
    status: Joi.number().required(),
    code: Joi.string().valid("API_CONNECTION_ERROR", "AUTHENTICATION_ERROR", "INVALID_DATA_ERROR", "DATABASE_ERROR", "NOT_FOUND_ERROR", "THIRD_PARTY_ERROR", "UNKNOWN_ERROR"),
    source: Joi.object({
        pointer: Joi.string().optional(),
        parameter: Joi.string().optional(),
        stack: Joi.string().optional()
    }).optional(),
    meta: Joi.object({
        environment: Joi.string().optional(),
        resource: Joi.string().optional(),
        url: Joi.string().optional(),
        httpMethod: Joi.string().optional(),
        functionName: Joi.string().optional()
    }).optional()
});
class JSONApiErrorStatic extends Error {
    constructor(error) {
        super(error ? error.details || error.message || "Unknown error" : "Unknown error");
        let exceptionErr;
        if (!error) {
            this.id = uuid.v1(),
                this.status = 500,
                this.code = ERROR_CODE_ENUM.UNKNOWN_ERROR,
                this.title = "Unknown error";
            return;
        }
        if (error instanceof JSONApiErrorStatic || JSONApiErrorStatic.isJSONApiError(error)) {
            // If the error is a JSONApiResponse
            if (!this.id) {
                this.id = uuid.v1();
                // assign it to this error and return
                for (const item in error) {
                    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                    //@ts-ignore
                    this[item] = error[item];
                }
            }
        }
        else if (error instanceof Joi.ValidationError) {
            this.id = uuid.v1();
            this.status = 400;
            this.title = "Validation error";
            this.message = error.message;
            this.code = ERROR_CODE_ENUM.INVALID_DATA_ERROR;
            this.source = { pointer: R.join("|", R.map(detail => detail.path, error.details)) };
            return;
        }
        else if (error instanceof Error) {
            exceptionErr = error;
            this.id = uuid.v1();
            this.status = 500;
            this.code = ERROR_CODE_ENUM.API_ERROR;
            this.title = exceptionErr.name;
        }
        else if (error) {
            const jsonError = error;
            this.id = jsonError.id || uuid.v1();
            this.status = jsonError.status || 500;
            this.code = jsonError.code || ERROR_CODE_ENUM.UNKNOWN_ERROR;
            this.title = jsonError.title || "Unknown error";
        }
        else {
            this.id = uuid.v1();
            this.status = 500;
            this.code = ERROR_CODE_ENUM.UNKNOWN_ERROR;
            this.title = "Unknown error";
        }
        if (["production", "staging"].includes(process.env.ENVIRONMENT) && [
            ERROR_CODE_ENUM.UNKNOWN_ERROR,
            ERROR_CODE_ENUM.DATABASE_ERROR,
            ERROR_CODE_ENUM.THIRD_PARTY_ERROR
        ].includes(this.code)) {
            Sentry.captureEvent({
                event_id: this.id,
                level: Sentry.Severity.Error,
                extra: {
                    error: {
                        stack: this.stack,
                        status: this.status,
                        code: this.code,
                        title: this.title,
                        source: this.source,
                        name: this.name,
                        meta: this.meta
                    },
                    server_name: process.env.SERVICE_NAME,
                    environment: process.env.ENVIRONMENT
                }
            });
        }
    }
    /**
     * Check if a plain object/ variable meets the JSONApiError compliance.
     *
     * @param error: the error object to check.
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    static isJSONApiError(error) {
        const { error: joiError } = exports.jsonAPIErrorJoiSchema.validate(error);
        return joiError ? false : true;
    }
    toJson() {
        return {
            id: this.id,
            status: this.status,
            code: this.code,
            title: this.title,
            details: this.message,
            meta: this.meta,
            source: this.source
        };
    }
}
exports.JSONApiErrorStatic = JSONApiErrorStatic;
/**
 * Creates and API
 *
 * Creates a JSON API error response
 *
 * @see http://jsonapi.org/
 * @param error - The suppose JSON API error object
 * @return A JSON API Response containing our error(s)
 */
const jsonApiError = (errorInfo) => {
    return new JSONApiErrorStatic(errorInfo);
};
exports.jsonApiError = jsonApiError;
// allowed error codes
var ERROR_CODE_ENUM;
(function (ERROR_CODE_ENUM) {
    ERROR_CODE_ENUM["API_ERROR"] = "API_ERROR";
    ERROR_CODE_ENUM["API_CONNECTION_ERROR"] = "API_CONNECTION_ERROR";
    ERROR_CODE_ENUM["AUTHENTICATION_ERROR"] = "AUTHENTICATION_ERROR";
    ERROR_CODE_ENUM["INVALID_DATA_ERROR"] = "INVALID_DATA_ERROR";
    ERROR_CODE_ENUM["DATABASE_ERROR"] = "DATABASE_ERROR";
    ERROR_CODE_ENUM["NOT_FOUND_ERROR"] = "NOT_FOUND_ERROR";
    ERROR_CODE_ENUM["THIRD_PARTY_ERROR"] = "THIRD_PARTY_ERROR";
    ERROR_CODE_ENUM["UNKNOWN_ERROR"] = "UNKNOWN_ERROR";
})(ERROR_CODE_ENUM = exports.ERROR_CODE_ENUM || (exports.ERROR_CODE_ENUM = {}));
